clear
close all
clc


% number of samples of each class
N = 100;
% define 2 clusters of input data
q = .6; % offset of classes
A = [rand(1,N/2)-q; rand(1,N/2)+q];
B = [rand(1,N/2)+q; rand(1,N/2)+q];
A = [A, [rand(1,N/2)+q; rand(1,N/2)-q]];
B = [B, [rand(1,N/2)-q; rand(1,N/2)-q]];


% plot
figure
plot(A(1,:),A(2,:),'xr')
hold on
plot(B(1,:),B(2,:),'o')
grid on
legend('A','B', 'C', 'D')
xlabel('x')
ylabel('y')
title('Input data')


%BREAK


% INPUT vector
P = [A, B];
% TARGET vector
tA = zeros(1, size(A,2));
tB = ones(1, size(A,2));
T = [tA, tB];


%BREAK


% create a neural network
neuronsXLayer = 10; % number of neurons per layer
neuronTopology{1} = 'tansig';


net = feedforwardnet(neuronsXLayer);

% training and testing data
net.divideParam.trainRatio = 0.5; 
net.divideParam.testRatio  = 0.5; 
net.divideParam.valRatio   = 0;

net.trainParam.epochs = 100;
net.trainParam.goal = 0.01;
for iL = 1: size(neuronsXLayer,2)
	net.layers{iL}.transferFcn = neuronTopology{iL}; 
end


%BREAK

% train a neural network
[net,tr,Y,E] = train(net,P,T);

%BREAK


% show network
view(net)

%BREAK

plotperform(tr)

%BREAK


% Training and testing data
trainP = P(:,tr.trainInd);
trainT = T(:,tr.trainInd);
trainResult = net(trainP);
trainResult = round(trainResult);

testP = P(:,tr.testInd);
testT = T(:,tr.testInd);
testResult = net(testP);
testResult = round(testResult);


%plot
figure,
plot(trainT)
hold on
plot(trainResult, '--r')
title('training results')

figure,
plot(testT)
hold on
plot(testResult, '--r')
title('testing results')

%BREAK


trainErr = trainT ~= trainResult;
testErr = testT ~= testResult;

figure,
plot(trainErr)
title('training error')

figure,
plot(testErr)
title('testing error')

%BREAK


% grid
coord = -1 : 0.01 : 2;
[Px,Py] = meshgrid(coord,coord);
pGrid = [Px(:) Py(:)]';

% simulate neural network on a grid
risGrid = net(pGrid);
F = TriScatteredInterp(Px(:), Py(:), risGrid');
risGridFull = F(Px, Py);


% plot classification regions
figure
imshow(risGridFull);
axis normal
colormap summer
hold on
plot((A(1,:)+1)*100,(A(2,:)+1)*100,'xr')
plot((B(1,:)+1)*100,(B(2,:)+1)*100,'o')
legend('A=0','B=1')
xlabel('x')
ylabel('y')
title('Output space')
colorbar


%BREAK

%Results
numTrainErr = size(find(trainErr > 0),2);
meanTrainErr = mean(trainErr);
stdTrainErr = std(trainErr);


numTestErr = size(find(testErr > 0),2);
meanTestErr = mean(testErr);
stdTestErr = std(testErr);


totalErr = [trainErr, testErr];
numTotalErr = size(find(totalErr > 0),2);
meanTotalErr = mean(totalErr);
stdTotalErr = std(totalErr);


fprintf('\nRESULTS \n')
fprintf('TRAINING\n')
fprintf('n. of el. = %i \n', size(trainErr,2))
fprintf('n. of errors = %i \n', numTrainErr)
fprintf('mean error = %f \n', meanTrainErr)
fprintf('std error = %f \n\n', stdTrainErr)
fprintf('TESTING\n')
fprintf('n. of el. = %i \n', size(testErr,2))
fprintf('n. of errors = %i \n', numTestErr)
fprintf('mean error = %f \n', meanTestErr)
fprintf('std error = %f \n\n', stdTestErr)
fprintf('TOTAL\n')
fprintf('n. of el. = %i \n', size(totalErr,2))
fprintf('n. of errors = %i \n', numTotalErr)
fprintf('mean error = %f \n', meanTotalErr)
fprintf('std error = %f \n\n', stdTotalErr)











