close all
clear all
clc

[x,t] = crab_dataset;
size(x)
size(t)

k = 10; % k-fold validation

P = x;
T = t(1,:);

TC = []; % T computed (results of NN)
TR = []; % T real(targets)


% create a neural network
neuronsXLayer = 2; % number of neurons per layer
neuronTopology{1} = 'tansig';


net = feedforwardnet(neuronsXLayer);

net.trainFcn = 'trainlm';  % Levenberg-Marquardt
net.performFcn = 'mse';  % Mean squared error

% k-fold validation is not automatic...
net.divideFcn = 'dividerand';  % Divide data randomly
net.divideMode = 'sample';  % Divide up every sample
net.divideParam.trainRatio =  1;
net.divideParam.valRatio =  0;
net.divideParam.testRatio = 0;

net.trainParam.epochs = 100;
net.trainParam.goal = 0.01;
for iL = 1: size(neuronsXLayer,2)
	net.layers{iL}.transferFcn = neuronTopology{iL}; 
end



%NEURAL NETWORK TRAINING AND K-FOLD VALIDATION
%__________________________________________________________________________
indices = crossvalind('Kfold', T ,k);

fprintf('%i-Fold Validation\n', k);

for i = 1: k
    
    fprintf(' k = %i\n', i);
    
    % indexes of the testing and training elements
    test1 = (indices == i);
    test = find(test1 > 0);
    traini1 = ~test1;
    traini = find(traini1 > 0);
    

    % train a neural network
    [net,tr,Y,E] = train(net,P(:, traini), T(traini));
    
    % show network
    if i == 1
        view(net)
    end
    
    
    % test
    testResultK = net(P(:,test));
    ind0 = find(testResultK < 0.5);
    testResultK(ind0) = 0;
    ind1 = find(testResultK >= 0.5);
    testResultK(ind1) = 1;
    
    % add elements to the global vectors TR and TC
    TR = [TR, T(test)];
    TC = [TC, testResultK];

end




testErr = TR ~= TC;



figure,
plot(testErr)
title('testing error')


%Results


numTestErr = size(find(testErr > 0),2);
meanTestErr = mean(testErr);
stdTestErr = std(testErr);


fprintf('\nRESULTS \n')

fprintf('n. of el. = %i \n', size(testErr,2))
fprintf('n. of errors = %i \n', numTestErr)
fprintf('mean error = %f \n', meanTestErr)
fprintf('std error = %f \n\n', stdTestErr)


% Confusion matrix
cm = confusionmat(TR,TC);
fprintf('Confusion matrix \n\n')
cm



