close all
clear all
clc


[x,t] = wine_dataset;
size(x)
size(t)

P = x;
T = (t(1,:)*2 + t(2,:));


% create a neural network
neuronsXLayer = 2; % number of neurons per layer
neuronTopology{1} = 'tansig';


net = feedforwardnet(neuronsXLayer);

% training and testing data
net.divideParam.trainRatio = 0.5; 
net.divideParam.testRatio  = 0.5; 
net.divideParam.valRatio   = 0;

net.trainParam.epochs = 100;
net.trainParam.goal = 0.01;
for iL = 1: size(neuronsXLayer,2)
	net.layers{iL}.transferFcn = neuronTopology{iL}; 
end


% train a neural network
[net,tr,Y,E] = train(net,P,T);

%BREAK


% show network
view(net)

%BREAK

plotperform(tr)

%BREAK


% Training and testing data
trainP = P(:,tr.trainInd);
trainT = T(:,tr.trainInd);
trainResult = net(trainP);
ind0 = find(trainResult < 0.5);
trainResult(ind0) = 0;
ind1 = find(trainResult >= 0.5 & trainResult < 1.5 );
trainResult(ind1) = 1;
ind2 = find(trainResult >= 1.5);
trainResult(ind2) = 2;

testP = P(:,tr.testInd);
testT = T(:,tr.testInd);
testResult = net(testP);
ind0 = find(testResult < 0.5);
testResult(ind0) = 0;
ind1 = find(testResult >= 0.5 & testResult < 1.5 );
testResult(ind1) = 1;
ind2 = find(testResult >= 1.5);
testResult(ind2) = 2;



trainErr = trainT ~= trainResult;
testErr = testT ~= testResult;


figure,
plot(trainErr)
title('training error')

figure,
plot(testErr)
title('testing error')


%Results
numTrainErr = size(find(trainErr > 0),2);
meanTrainErr = mean(trainErr);
stdTrainErr = std(trainErr);


numTestErr = size(find(testErr > 0),2);
meanTestErr = mean(testErr);
stdTestErr = std(testErr);


totalErr = [trainErr, testErr];
numTotalErr = size(find(totalErr > 0),2);
meanTotalErr = mean(totalErr);
stdTotalErr = std(totalErr);


fprintf('\nRESULTS \n')
fprintf('TRAINING\n')
fprintf('n. of el. = %i \n', size(trainErr,2))
fprintf('n. of errors = %i \n', numTrainErr)
fprintf('mean error = %f \n', meanTrainErr)
fprintf('std error = %f \n\n', stdTrainErr)
fprintf('TESTING\n')
fprintf('n. of el. = %i \n', size(testErr,2))
fprintf('n. of errors = %i \n', numTestErr)
fprintf('mean error = %f \n', meanTestErr)
fprintf('std error = %f \n\n', stdTestErr)
fprintf('TOTAL\n')
fprintf('n. of el. = %i \n', size(totalErr,2))
fprintf('n. of errors = %i \n', numTotalErr)
fprintf('mean error = %f \n', meanTotalErr)
fprintf('std error = %f \n\n', stdTotalErr)

% Confusion matrix
cm = confusionmat(testT,testResult);
fprintf('Confusion matrix \n\n')
cm
